#!/bin/sh
# $Id: configure.sh,v 1.8 2004/12/08 02:40:53 jimb Exp $
# Author   : Jim Brooks http://www.jimbrooks.org
# Synopsis : Simple configuration script.
#            Mainly intended for configuration on Linux or FreeBSD.
# Notes    : gtk[glib]-config is named gtk12[glib12]-config on FreeBSD 4.4.
# --------------------------------------------------------------------------

# gtk-config or gtk12-config?
# First look for gtk12-config (in case gtk-config is for > GTK+ 1.2.x).
#
GTK_CONFIG=gtk12-config
GLIB_CONFIG=glib12-config

$GTK_CONFIG --version >/dev/null 2>&1
if [ "$?" != "0" ]
then
	GTK_CONFIG=gtk-config
	GLIB_CONFIG=glib-config

	$GTK_CONFIG --version >/dev/null 2>&1
	if [ "$?" != "0" ]
	then
		echo "-- ERROR: GTK+ 1.2.x is not installed."
		exit 1
	fi
fi

# Create config file to be included by GNUmakefile.
#
echo "#--- AUTO-GENERATED by configure.sh on `uname` ---"  > GNUmakefile.config
echo "GTK_CONFIG=${GTK_CONFIG}"                           >> GNUmakefile.config
echo "GLIB_CONFIG=${GLIB_CONFIG}"                         >> GNUmakefile.config

# Depending on system, determine name of GNU make and create a "makefile".
#
if [ "`uname -a | grep Linux`" != "" ]
then
	# Linux.
	#
	GMAKE=make
else
	# BSD or not Linux.
	#
	GMAKE=gmake

	# Create a traditional "makefile" which defers to GNU make.
	#
	echo "#--- AUTO-GENERATED by configure.sh on `uname` ---"  > makefile
	echo "all .DEFAULT:"                                      >> makefile
	# This echo starts with a TAB (echo -e doesn't work on Solaris).
	echo "	${GMAKE} -f GNUmakefile \$@"                      >> makefile
fi

# Ensure GNU make is installed.
#
if [ "`$GMAKE --version 2>/dev/null | grep GNU`" = "" ]
then
	echo "-- ERROR: GNU make is not installed."
	exit 1
fi

# Check if Exuberant ctags is installed, but if it isn't just warn.
#
FOUND_CTAGS=0
FindCtags()
{
    if [ "$FOUND_CTAGS" = "0" ]; then
        CTAGS=$1
        if ( which $CTAGS >/dev/null 2>&1 ); then
            FOUND_CTAGS=1
        elif ( $CTAGS --help 2>&1 | grep Exuberant >/dev/null ); then
            FOUND_CTAGS=1
        fi
    fi
}
FindCtags ctags
FindCtags exuberant-ctags
FindCtags exctags
if [ "$FOUND_CTAGS" = "0" ]; then
    echo "-- Warning:"
    echo "-- hypersrc depends on Exuberant ctags which doesn't appear to be installed"
    echo "-- (or GNU ctags precedes it in PATH?)"
	echo "-- Exuberant ctags is available at:"
	echo "-- http://ctags.sourceforge.net or ftp.jimbrooks.org/hypersrc/exuberant_ctags/"
fi

# Done.
#
echo "Configured for `uname`."
echo "Now do:"
echo "make install"

exit 0
